/*
 * Copyright (C) 2007-2010 Collabora Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Xavier Claessens <xclaesse@gmail.com>
 *          Guillaume Desmottes <guillaume.desmottes@collabora.co.uk>
 */

/* Helper functions to automatically create a Salut account */

#ifndef __AUTO_SALUT_ACCOUNT_HELPER_H__
#define __AUTO_SALUT_ACCOUNT_HELPER_H__

#include <telepathy-glib/account-manager.h>

#include <libempathy/empathy-connection-managers.h>
#include <libempathy/empathy-account-settings.h>

gboolean should_create_salut_account (TpAccountManager *manager);

EmpathyAccountSettings * create_salut_account_settings (void);

#endif
