/* Auto-generated, do not edit.
 *
 * This file may be distributed under the same terms
 * as the specification from which it was generated.
 */

#define TPY_HASH_TYPE_CONTACT_CODEC_MAP (tpy_type_dbus_hash_ua_28usuua_7bss_7d_29 ())

#define TPY_HASH_TYPE_CONTACT_SENDING_STATE_MAP (tpy_type_dbus_hash_uu ())

#define TPY_HASH_TYPE_CANDIDATE_INFO (tpy_type_dbus_hash_sv ())

#define TPY_HASH_TYPE_CALL_MEMBER_MAP (tpy_type_dbus_hash_uu ())

#define TPY_ARRAY_TYPE_CALL_MEMBER_MAP_LIST (tpy_type_dbus_array_of_a_7buu_7d ())

GType tpy_type_dbus_hash_uu (void);

GType tpy_type_dbus_hash_ua_28usuua_7bss_7d_29 (void);

GType tpy_type_dbus_hash_sv (void);

#define TPY_STRUCT_TYPE_CODEC (tpy_type_dbus_struct_usuua_7bss_7d ())

#define TPY_ARRAY_TYPE_CODEC_LIST (tpy_type_dbus_array_usuua_7bss_7d ())

#define TPY_STRUCT_TYPE_CODEC_OFFERING (tpy_type_dbus_struct_oua_28usuua_7bss_7d_29 ())

#define TPY_STRUCT_TYPE_VIDEO_RESOLUTION (tpy_type_dbus_struct_uu ())

#define TPY_ARRAY_TYPE_VIDEO_RESOLUTION_STRUCT (tpy_type_dbus_array_uu ())

#define TPY_STRUCT_TYPE_CANDIDATE (tpy_type_dbus_struct_usua_7bsv_7d ())

#define TPY_ARRAY_TYPE_CANDIDATE_LIST (tpy_type_dbus_array_usua_7bsv_7d ())

#define TPY_STRUCT_TYPE_STREAM_CREDENTIALS (tpy_type_dbus_struct_ss ())

#define TPY_STRUCT_TYPE_CALL_STATE_REASON (tpy_type_dbus_struct_uus ())

GType tpy_type_dbus_struct_usua_7bsv_7d (void);

GType tpy_type_dbus_struct_uu (void);

GType tpy_type_dbus_struct_oua_28usuua_7bss_7d_29 (void);

GType tpy_type_dbus_struct_uus (void);

GType tpy_type_dbus_struct_usuua_7bss_7d (void);

GType tpy_type_dbus_struct_ss (void);

GType tpy_type_dbus_array_usuua_7bss_7d (void);

GType tpy_type_dbus_array_uu (void);

GType tpy_type_dbus_array_usua_7bsv_7d (void);

GType tpy_type_dbus_array_of_a_7buu_7d (void);

